@@ -7,7 +7,7 @@ from typing import Callable, Iterable, Literal, Sequence, cast, overload
 
 from vsaa import Eedi3, Nnedi3, SuperSampler
 from vskernels import Catrom, Kernel, KernelT, Scaler, ScalerT, Spline144
-from vsmasktools import EdgeDetect, EdgeDetectT, Prewitt
+from vsmasktools import Prewitt, GenericMaskT, normalize_mask
 from vstools import (
     CustomValueError, FieldBased, FieldBasedT, FuncExceptT, check_variable, core, depth, get_depth, get_h, get_prop,
     get_w, get_y, join, normalize_seq, split, vs
@@ -16,7 +16,7 @@ from vstools import (
 from .helpers import scale_var_clip
 from .mask import descale_detail_mask
 from .scale import SSIM
-from .types import CreditMaskT, DescaleAttempt, DescaleMode, DescaleResult, PlaneStatsKind, _DescaleTypeGuards
+from .types import DescaleAttempt, DescaleMode, DescaleResult, PlaneStatsKind
 
 __all__ = [
     'get_select_descale', 'descale',
@@ -30,6 +30,7 @@ __all__ = [
 def get_select_descale(
     clip: vs.VideoNode, descale_attempts: list[DescaleAttempt], mode: DescaleMode
 ) -> tuple[Callable[[list[vs.VideoFrame], int], vs.VideoNode], list[vs.VideoNode]]:
+    """Get callables for FrameEval/ModifyFrame and prop clips for the specified params."""
     clips_by_reskern = {
         attempt.da_hash: attempt
         for attempt in descale_attempts
@@ -143,7 +144,9 @@ def descale(  # type: ignore
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
     kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT | None = Nnedi3,
-    mask: CreditMaskT | Literal[False] = descale_detail_mask,
+    mask: GenericMaskT | Literal[False] | tuple[
+        GenericMaskT | Literal[False] | None, GenericMaskT
+    ] = descale_detail_mask,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
     shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
@@ -152,68 +155,14 @@ def descale(  # type: ignore
     ...
 
 
-@overload
-def descale(  # type: ignore
-    clip: vs.VideoNode,
-    width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT = Nnedi3,
-    mask: CreditMaskT = descale_detail_mask,
-    mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
-    dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
-    result: Literal[True] = True
-) -> _DescaleTypeGuards.UpscalerNotNoneMaskNotNone:
-    ...
-
-
-@overload
-def descale(  # type: ignore
-    clip: vs.VideoNode,
-    width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT = Nnedi3,
-    mask: Literal[False] = False,
-    mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
-    dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
-    result: Literal[True] = True
-) -> _DescaleTypeGuards.UpscalerNotNoneMaskIsNone:
-    ...
-
-
-@overload
-def descale(  # type: ignore
-    clip: vs.VideoNode,
-    width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: None = None,
-    mask: CreditMaskT = descale_detail_mask,
-    mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
-    dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
-    result: Literal[True] = True
-) -> _DescaleTypeGuards.UpscalerIsNoneMaskNotNone:
-    ...
-
-
-@overload
-def descale(
-    clip: vs.VideoNode,
-    width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
-    kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: None = None,
-    mask: Literal[False] = False,
-    mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
-    dst_width: int | None = None, dst_height: int | None = None,
-    shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
-    result: Literal[True] = True
-) -> _DescaleTypeGuards.UpscalerIsNoneMaskIsNone:
-    ...
-
-
 @overload
 def descale(
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
     kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT | None = Nnedi3,
-    mask: CreditMaskT | Literal[False] = descale_detail_mask,
+    mask: GenericMaskT | Literal[False] | tuple[
+        GenericMaskT | Literal[False] | None, GenericMaskT
+    ] = descale_detail_mask,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
     shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
@@ -226,12 +175,106 @@ def descale(
     clip: vs.VideoNode,
     width: int | Iterable[int] | None = None, height: int | Iterable[int] = 720,
     kernels: KernelT | Sequence[KernelT] = Catrom, upscaler: ScalerT | None = Nnedi3,
-    mask: CreditMaskT | Literal[False] = descale_detail_mask,
+    mask: GenericMaskT | Literal[False] | tuple[
+        GenericMaskT | Literal[False] | None, GenericMaskT
+    ] = descale_detail_mask,
     mode: DescaleMode = DescaleMode.PlaneDiff(0.0),
     dst_width: int | None = None, dst_height: int | None = None,
     shift: tuple[float, float] = (0, 0), scaler: ScalerT = Spline144,
     result: bool = False
 ) -> vs.VideoNode | DescaleResult:
+    """
+    A unified and expansive descaling function, includes support for handling
+    fractional resolutions, multi-res, FHD detail masking, and conditional descaling.
+
+    .. warning::
+
+        Only descale if you are absolutely certain you know the correct native resolution(s) and kernel(s)!
+        Wrong rescales will be more destructive than simply AAing/dehaloing instead!
+
+    ``width`` and ``height`` both accept multiple values, allowing you to descale to multiple resolutions
+    in case your source is upscaled from different native resolutions. If ``width`` is not set,
+    it will be auto-calculated using the clip's aspect ratio and given height(s) as reference.
+
+    You can pass a list of :py:class:`vskernels.Kernel` objects, allowing you to handle mixed sources.
+    Common examples include shows that have been upscaled using either SharpBicubic or Spline36 throughout its runtime.
+    If you find yourself working on a SharpBicubic, Spline36, or Lanczos source,
+    consider double-checking other scenes and episodes for any of the other kernels mentioned.
+    **Especially** if you've got either SharpCubic or Spline36!
+
+    For fractional resolution descaling, you can pass the ``src_width`` and ``src_height`` to the kernel(s)
+    passed to the function.
+
+    Conditional descaling can be achieved by making use of ``mode``. Setting a specific mode determines
+    how the condition is applied, and a threshold can be set. If the difference between the descaled
+    and re-upscaled clip (using the same kernel) exceeds the threshold, it will not descale the frame.
+    For more information, please refer to :py:class:`DescaleMode`'s docstring.
+
+    Custom upscalers can be passed as well. If you'd like to write a custom upscaler, make sure
+    you create a class with a ``scale`` method that adheres to the following API:
+
+    ```py
+    class.scale(clip, width, height)
+    ```
+
+    All the results can be returned using ``result``. This will return a :py:class:`DescaleResult` object
+    containing the following attributes:
+
+    * ``descaled`` (descaled clip, can be a variable-resolution clip)
+    * ``rescaled`` (rescaled clip, can be a variable-resolution clip)
+    * ``upscaled`` (upscaled clip)
+    * ``mask`` (Descale error mask)
+    * ``attempts`` (List of :py:class:`DescaleAttempt`s)
+    * ``out`` (Final rescaled, masked, and chroma-merged clip)
+
+    This has multiple applications, like for example needing the mask for post-rescaling masking of credits
+    (like for post-AA, for example), or if you want to do additional filtering while it's descaled.
+
+    :param clip:            Clip to process.
+    :param width:           Width to descale to. Pass a list if your source has multiple native resolutions.
+                            If None, auto-calculates based on the clip's AR and given height(s).
+                            For fractional resolutions, set ``src_width`` in the :py:class:`vskernels.Kernel`.
+                            Make sure you're confident about the native resolution(s) before descaling!
+                            Default: None.
+    :param height:          Height to descale to. Pass a list if your source has multiple native resolutions.
+                            For fractional resolutions, set ``src_height`` in the :py:class:`vskernels.Kernel`.
+                            Make sure you're confident about the native resolution(s) before descaling!
+                            Default: 720.
+    :param kernels:         :py:class:`vskernels.Kernel`s to descale to. Pass a list to enable multi-kernel descaling.
+                            When passing multiple kernels, make sure you use a :py:class:`DescaleMode` that supports it.
+                            Default: :py:class:`vskernels.Catrom`.
+    :param upscaler:        Scaler used for rescaling. A passing scaler can be passed, but it must be a class with
+                            a ``scale`` method that accepts a `clip, width, height`.
+                            Setting this to False will disable upscaling entirely.
+                            Default: :py:class:`vsaa.Nnedi3`.
+    :param mask:            Internal masking function. This can be either a custom mask or a masking function.
+                            Setting this to False will disable masking entirely.
+                            Default: :py:func:`vsscale.mask.descale_detail_mask`.
+    :param mode:            :py:class:`DescaleMode` used to determine how conditional descaling is handled.
+                            For more information, please refer to :py:class:`DescaleMode`'s docstring.
+                            Default: :py:func:`DescaleMode.PlaneDiff(0.0)`.
+    :param dst_width:       Destination width. Set the clip's output width.
+                            If None and a ``dst_height`` is passed, auto-calculate based on
+                            the clip's AR and given ``dst_height``. Else, use the input clip's width.
+                            Default: None.
+    :param dst_height:      Destination height. Set the clip's output height.
+                            If None, use the input clip's height.
+                            Default: None.
+    :param shift:           Shift the clip. Useful for cross-conversions, or if the image was shifted in the upscale.
+                            Default: (0, 0), no shifting.
+    :param scaler:          Scaler used for operations where regular scaling is required (for example,
+                            ``upscaler=None``, for scaling chroma and descale detail mask).
+                            Default: :py:class:`vskernels.Spline144`
+    :param result:          Return the :py:class:`DescaleResult` object.
+                            If False, return the regularly rescaled output.
+                            Default: False.
+
+    :raises ValueError:     Number of given heights and width don't match.
+    :raises ValueError:     No kernel is specified.
+
+    :returns:               Either a rescaled clip (mask applied, chroma readded),
+                            or a :py:class:`DescaleResult` object containing the results from ``descale``.
+    """
     assert clip.format
 
     if isinstance(height, int):
@@ -321,20 +364,27 @@ def descale(
         )
         upscaled = depth(upscaled, bit_depth)
 
-    if mask:
-        if isinstance(mask, EdgeDetectT):  # type: ignore
-            mask = EdgeDetect.ensure_obj(mask).edgemask(clip_y)  # type: ignore
-        elif callable(mask):
-            mask = mask(clip, rescaled)  # type: ignore
+    if isinstance(mask, tuple):
+        error_mask, pproc_mask = mask
+    else:
+        error_mask, pproc_mask = mask, None
 
-        assert isinstance(mask, vs.VideoNode)
+    if upscaled:
+        clip_y = scaler.scale(clip_y, dest_width, dest_height)
 
-        mask = depth(mask, bit_depth)  # type: ignore
+        if error_mask:
+            error_mask = normalize_mask(error_mask, clip_y, upscaled)
+            error_mask = scaler.scale(error_mask, dest_width, dest_height)
 
-        if upscaled:
-            mask = scaler.scale(mask, dest_width, dest_height)
-            clip_y = scaler.scale(clip_y, dest_width, dest_height)
-            upscaled = upscaled.std.MaskedMerge(clip_y, mask)
+            upscaled = upscaled.std.MaskedMerge(clip_y, error_mask)
+
+        if pproc_mask:
+            pproc_mask = normalize_mask(pproc_mask, clip_y, upscaled)
+            pproc_mask = scaler.scale(pproc_mask, dest_width, dest_height)
+
+            upscaled = clip_y.std.MaskedMerge(upscaled, pproc_mask)
+    else:
+        error_mask = pproc_mask = None
 
     if upscaled:
         out = upscaled
@@ -346,9 +396,11 @@ def descale(
     if chroma and upscaled and (clip.width, clip.height) == (dest_width, dest_height):
         out = join([upscaled, *chroma], clip.format.color_family)
 
+    out = out.std.CopyFrameProps(clip)
+
     if result:
         return DescaleResult(
-            descaled, rescaled, upscaled, mask if mask else None, descale_attempts, out  # type: ignore
+            descaled, rescaled, upscaled, error_mask, pproc_mask, descale_attempts, out
         )
 
     return out
@@ -357,9 +409,9 @@ def descale(
 def mixed_rescale(
     clip: vs.VideoNode, width: None | int = None, height: int = 720,
     kernel: KernelT = Catrom, downscaler: ScalerT = SSIM,
-    credit_mask: vs.VideoNode | CreditMaskT | bool = partial(descale_detail_mask, thr=0.05, inflate=4, xxpand=(4, 2)),
+    credit_mask: vs.VideoNode | GenericMaskT | bool = partial(descale_detail_mask, thr=0.05, inflate=4, xxpand=(4, 2)),
     mix_strength: float = 0.25, show_mask: bool | int = False,
-    # Default settings set to match insaneAA as closely as reasonably possible
+    # Default settings set to match insaneAA as closely and as reasonably possible
     eedi3: SuperSampler = Eedi3(
         alpha=0.2, beta=0.25, gamma=1000, nrad=2, mdis=20, sclip_aa=Nnedi3(nsize=0, nns=4, qual=2, pscrn=1)
     )
@@ -418,16 +470,9 @@ def mixed_rescale(
     merged = core.akarin.Expr([descaled, downscaled], f'x {mix_strength} * y 1 {mix_strength} - * +')
 
     if credit_mask:
-        if isinstance(credit_mask, vs.VideoNode):
-            detail_mask = depth(credit_mask, get_depth(clip))  # type: ignore
-        elif callable(credit_mask) and not isinstance(credit_mask, type):
-            detail_mask = credit_mask(clip_y, upscaled)
-        else:
-            detail_mask = EdgeDetect.ensure_obj(
-                Prewitt if credit_mask is True else credit_mask  # type: ignore
-            ).edgemask(merged)
-
-        detail_mask = detail_mask.std.Limiter()
+        detail_mask = normalize_mask(
+            Prewitt if credit_mask is True else credit_mask, clip_y, upscaled
+        ).std.Limiter()
     else:
         detail_mask = None
 
